import os
import math
import requests
import json
import datetime
import hashlib
import dateparser
import re
import csv
from pygdpr.models.dpa import DPA
from bs4 import BeautifulSoup
from pygdpr.services.filename_from_path_service import filename_from_path_service
from pygdpr.services.pdf_to_text_service import PDFToTextService
from pygdpr.specifications import pdf_file_extension_specification
from pygdpr.specifications.should_retain_document_specification import ShouldRetainDocumentSpecification
from pygdpr.models.common.pagination import Pagination
from pygdpr.policies.gdpr_policy import GDPRPolicy
import textract
import docx2txt
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from pygdpr.policies.webdriver_exec_policy import WebdriverExecPolicy
from selenium.webdriver.common.keys import Keys
import time

class Lithuania(DPA):
    def __init__(self, path=os.curdir):
        country_code='lt'
        super().__init__(country_code, path)

    def update_pagination(self, pagination=None, page_soup=None, driver=None):
        source = {
            "host": "https://vdai.lrv.lt",
            "start_path": "/lt/naujienos/exportPublicData?export_data_type=csv&download=1"
        }
        host = source["host"]
        start_path = source["start_path"]
        if pagination is None:
            pagination = Pagination()
            pagination.add_item(host + start_path)
        return pagination

    def get_source(self, page_url=None, driver=None, to_print=True):
        assert (page_url is not None)
        results_response = None
        session = requests.Session()
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:98.0) Gecko/20100101 Firefox/98.0",
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,*/*;q=0.8",
            "Accept-Language": "en-US,en;q=0.5",
            "Accept-Encoding": "gzip, deflate",
            "Connection": "keep-alive",
            "Upgrade-Insecure-Requests": "1",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "none",
            "Sec-Fetch-User": "?1",
            "Cache-Control": "max-age=0",
         }
        try:
            results_response = session.get(page_url, headers=headers)
            # results_response = requests.get(page_url, headers=headers, timeout=10)
            # results_response = requests.request('GET', page_url)
            results_response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            if to_print:
                print(error)
            pass
        return results_response

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        added_docs = []
        # call all the get_docs_X() functions
        #added_docs += self.get_docs_Decisions(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Guidelines(existing_docs=[], overwrite=False, to_print=True)
        # added_docs += self.get_docs_InspectionReports(existing_docs=[], overwrite=False, to_print=True)
        return added_docs

    def get_docs_Decisions(self, existing_docs=[], overwrite=False, to_print=True):
        existed_docs = []
        source = {
            "host": "https://vdai.lrv.lt",
            "start_path": "/lt/naudinga-informacija/vdai-sprendimai-baudos-nurodymai-ir-kt"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        if to_print:
            print('Page:\t', page_url)

        # TODO: HTTPError: 503 Server Error: Service Temporarily Unavailable for url
        page_source = self.get_source(page_url=page_url)

        # try to use selenium, but failed
        # exec_path = WebdriverExecPolicy().get_system_path()
        # options = webdriver.ChromeOptions()
        # options.add_argument('headless')
        # driver_doc = webdriver.Chrome(options=options, executable_path=exec_path)
        # driver_doc.get(page_url)
        # print(driver_doc.page_source)

        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        content = results_soup.find('div', class_='content text')
        year_list = []
        for p in content.find_all('p'):
            potential_year = p.get_text().strip()
            if potential_year.isdigit():
                year = potential_year
                year_list.append(year)
            document_section = p.find('a')
            if document_section is None:
                continue
            for documents in p.find_all('a'):
                document_href = documents.get('href')
                document_title = documents.get_text()
                if not document_href.endswith('pdf'):
                    continue
                document_url = host + document_href
                #print('document_url: ', document_url)
                print('document_title: ', document_title)
                date_str = document_title.split()[-1]

                # check whether the title contain date
                # if not, use the default date: year-01-01
                if not date_str[0].isdigit():
                    date_str = year_list[-1] + '-01-01'
                else:
                    date_str = date_str
                tmp = datetime.datetime.strptime(date_str, '%Y-%m-%d')
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                print('\tdate: ', date)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                if to_print:
                    print("\tDocument:\t", document_hash)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Decisions' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                except FileExistsError:
                    pass
                with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                    f.write(document_content)
                with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                    document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                    f.write(document_text)
                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                    metadata = {
                        'title': {
                            self.language_code: document_title
                        },
                        'md5': document_hash,
                        'releaseDate': date.strftime('%d/%m/%Y'),
                        'url': document_url
                    }
                    json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                existed_docs.append(document_hash)
        return existed_docs

    def get_docs_Guidelines(self, existing_docs=[], overwrite=False, to_print=True):
        existed_docs = []
        source = {
            "host": "https://vdai.lrv.lt",
            "start_path": "/lt/naudinga-informacija/rekomendacijos-gaires-ir-kt"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        if to_print:
            print('Page:\t', page_url)
        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        panel = results_soup.find('div', class_='panel-group')
        for p in panel.find_all('p'):
            for documents in p.find_all('a'):
                time.sleep(5)
                document_href = documents.get('href')
                document_title = documents.get_text()
                if not document_href.endswith('pdf') and not document_href.endswith('docx'):
                    continue
                if document_href.startswith('https'):
                    document_url = document_href
                else:
                    document_url = host + document_href
                print('document_title: ', document_title)
                print('\tdocument_url: ', document_url)
                # get the date from the document_url
                potential_date_str = ''
                for i in range(len(document_url)):
                    if document_url[i].isdigit() or document_url[i] == '-':
                        potential_date_str += document_url[i]
                if len(potential_date_str) > 10:
                    potential_date_str = potential_date_str[-10:]
                # add the default day and month if needed to complete the date format
                if len(potential_date_str) == 0:
                    potential_date_str = '2018-05-25'
                elif len(potential_date_str) == 4:  # only contain year
                    potential_date_str = potential_date_str + '-05-25'
                elif len(potential_date_str) == 7:  # only contain year and month
                    potential_date_str = potential_date_str + '-25'
                elif '-' not in potential_date_str:  # only contain digit without '-'
                    potential_date_str = potential_date_str[-8:]
                    potential_date_str = potential_date_str[:4] + '-' + potential_date_str[4:6] + '-' + potential_date_str[6:]

                date_str = potential_date_str
                print('\tdate_str: ', date_str)
                tmp = datetime.datetime.strptime(date_str, '%Y-%m-%d')
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                print('\tdate: ', date)
                # In this link, all files are sorted from newest to oldest.
                # So just break the loop if one file not satisfied by date
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    print('all the files after will not satisfied by date')
                    return existed_docs
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                if to_print:
                    print("\tDocument:\t", document_hash)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Guidelines' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                except FileExistsError:
                    pass
                if document_url.endswith('.docx'):
                    # set a download path. Use selenium to download it
                    exec_path = WebdriverExecPolicy().get_system_path()
                    options = webdriver.ChromeOptions()
                    prefs = {"download.default_directory": document_folder}
                    options.add_argument('headless')
                    options.add_experimental_option("prefs", prefs)
                    driver_doc = webdriver.Chrome(options=options, executable_path=exec_path)
                    # open google search, and put the document_url into search box, and press enter
                    driver_doc.get('https://www.google.com/')
                    box = driver_doc.find_element_by_xpath(  # google search box xpath
                        '/html/body/div[1]/div[3]/form/div[1]/div[1]/div[1]/div/div[2]/input')
                    box.send_keys(document_url)
                    box.send_keys(Keys.ENTER)
                    # the first result in the search is the file we need. Click the link to download it
                    download_section = driver_doc.find_element_by_class_name('tF2Cxc')
                    button = download_section.find_element_by_tag_name('h3').click()
                    time.sleep(5)
                    files = os.listdir(document_folder)
                    for index, file in enumerate(files):
                        os.rename(os.path.join(document_folder, file),
                                  os.path.join(document_folder, self.language_code + '.docx'))
                    document_content = docx2txt.process(document_folder + '/' + self.language_code + '.docx')
                    with open(
                            document_folder + '/' + self.language_code + '.txt', 'w') as f:
                        f.write(document_content)
                else:
                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)
                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                    metadata = {
                        'title': {
                            self.language_code: document_title
                        },
                        'md5': document_hash,
                        'releaseDate': date.strftime('%d/%m/%Y'),
                        'url': document_url
                    }
                    json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                existed_docs.append(document_hash)
        return existed_docs

    def get_docs_InspectionReports(self, existing_docs=[], overwrite=False, to_print=True):
        existed_docs = []
        source = {
            "host": "https://vdai.lrv.lt",
            "start_path": "/lt/naudinga-informacija/patikrinimu-rezultatu-apibendrinimai"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        if to_print:
            print('Page:\t', page_url)

        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        body = results_soup.find('div', class_='panel-body text')
        for p in body.find_all('p'):
            for documents in p.find_all('a'):
                document_href = documents.get('href')
                document_title = documents.get_text().splitlines()[-1]
                if not document_href.endswith('pdf') and not document_href.endswith('docx'):
                    continue
                document_url = host + document_href
                print('document_title: ', document_title)
                print('document_url: ', document_url)
                potential_date_str = document_url.split('.')[-2]
                # get the date from the url
                potential_date_str = potential_date_str.split('.')[0]
                potential_date_str = potential_date_str[-10:]

                # check whether the date_str is vaild date
                date_str = ''
                for i in range(len(potential_date_str)):
                    if potential_date_str[i].isdigit() or potential_date_str[i] == '-':
                        date_str += potential_date_str[i]
                # some date_str need to add the '-' to have a correct format
                if len(date_str) == 8:
                    date_str = date_str[0:4] + '-' + date_str[4:6] + '-' + date_str[6:]
                tmp = datetime.datetime.strptime(date_str, '%Y-%m-%d')
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                print('\tdate: ', date)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Inspection Reports' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                except FileExistsError:
                    pass
                if to_print:
                    print("\tDocument:\t", document_hash)
                if document_url.endswith('.docx'):
                    # set a download path. Use selenium to download it
                    exec_path = WebdriverExecPolicy().get_system_path()
                    options = webdriver.ChromeOptions()
                    prefs = {"download.default_directory": document_folder}
                    options.add_argument('headless')
                    options.add_experimental_option("prefs", prefs)
                    driver_doc = webdriver.Chrome(options=options, executable_path=exec_path)
                    # open google search, and put the document_url into search box, and press enter
                    driver_doc.get('https://www.google.com/')
                    box = driver_doc.find_element_by_xpath('/html/body/div[1]/div[3]/form/div[1]/div[1]/div[1]/div/div[2]/input')
                    box.send_keys(document_url)
                    box.send_keys(Keys.ENTER)
                    # the first result in the search is the file we need. Click the link to download it
                    download_section = driver_doc.find_element_by_class_name('tF2Cxc')
                    button = download_section.find_element_by_tag_name('h3').click()
                    time.sleep(5)
                    files = os.listdir(document_folder)
                    for index, file in enumerate(files):
                        os.rename(os.path.join(document_folder, file), os.path.join(document_folder, self.language_code + '.docx'))
                    document_content = docx2txt.process(document_folder + '/' + self.language_code + '.docx')
                    with open(
                            document_folder + '/' + self.language_code + '.txt', 'w') as f:
                        f.write(document_content)
                else:
                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)
                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                    metadata = {
                        'title': {
                            self.language_code: document_title
                        },
                        'md5': document_hash,
                        'releaseDate': date.strftime('%d/%m/%Y'),
                        'url': document_url
                    }
                    json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                existed_docs.append(document_hash)
        return existed_docs









from ..specifications.join_lines_specification import JoinLinesSpecification

class JoinLinesPDFToTextService():
    """
    A service class which joins broken up lines of text (eg. a newline mistakenly introduced midsentence).

    Methods
    -------
    seek_next(j, lines)
        Returns the next line with a length greater than 0 (None if none found).

    for_text(text)
        Returns a modified copy of the text with no broken up lines.
    """
    def seek_next(self, j, lines):
        """Returns the next line with a length greater than 0 (None if none found).

        Parameters
        ----------
        j : int
            The iterator
        lines : [str]
            The list of lines.

        Returns
        -------
        str
            the next line with a length greater than 0 (None if none found)
        """
        next_line = None
        while next_line is None and j < len(lines):
            if len(lines[j]) > 0:
                next_line = lines[j]
                break
            j += 1
        return next_line

    def for_text(self, text):
        """Returns a modified copy of the text with no broken up lines.

        Parameters
        ----------
        text : str
            The text with broken up lines.

        Returns
        -------
        str
            a modified copy of the text parameter with no broken up lines
        """
        if len(text) == 0:
            return ""
        text_mod = ""
        lines = text.split("\n")
        i = 0
        for i in range(len(lines)):
            line = lines[i]
            j = i+1
            next_line = self.seek_next(j, lines)
            if next_line is None:
                break
            print('line:', line)
            print('next_line:', next_line)
            if JoinLinesSpecification().is_satisfied_by(line, next_line):
                text_mod += ' ' + next_line
            else:
                text_mod += '\n'.join(lines[i:j+1])
            i = j
        return text_mod



import os
import json
from pygdpr.models.common.eu_member import EUMember

class ListEUMembersService():
    def get(self):
        eu_members = []
        path = os.path.abspath('pygdpr/assets/eu-members.json')
        f = open(path, 'r')
        eu_members_items = json.load(f)
        f.close()
        for country_code, country_name, in eu_members_items.items():
            eu_member = EUMember(country_code)
            eu_members.append(eu_member)
        return eu_members



from setuptools import setup, find_packages
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()
setup(
    name = 'gdprCrawlerTest20',
    version = '0.0.1',
    author = 'Chen Sun, Supreeth Shastri, Evan Jacobs',
    author_email = 'chen-sun@uiowa.edu',
    license = 'MIT License',
    description = 'GDPR document crawler',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = 'https://github.com/transientCloud/gdpr-sota',
    py_modules = ['gdprCrawler'],
    packages = find_packages(),
    include_package_data=True,
    install_requires = [requirements],
    python_requires='>=3.9',
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "Operating System :: MacOS",
    ],
    entry_points = '''
        [console_scripts]
        gdprCrawler=gdprCrawler:cli
    '''
)



class Pagination(object):
    def __init__(self):
        self.links = []
        self.cursor = 0

    def add_item(self, link):
        if link in self.links:
            return
        self.links.append(link)

    def reset_next(self):
        self.cursor = 0

    def has_next(self):
        return self.cursor <= len(self.links) - 1

    # Check if object has specific page link
    def has_link(self, link):
        if link in self.links:
            return True
        else:
            return False

    def get_next(self):
        if self.has_next() is False:
            raise ValueError('Pagination.get_next() cursor is out of bounds')

        link = self.links[self.cursor]
        self.cursor += 1
        return link

    def is_empty(self):
        return len(self.links) == 0



from pygdpr.services.metadata_service import *
import nltk

class StatisticsMetadataService(MetadataService):
    def size_in_bytes(self, text):
        size_in_bytes = len(text.encode('utf-8'))
        return size_in_bytes

    def char_count(self, text):
        chars = 0
        for c in text:
            if c == " " or c == "\s": continue
            chars += 1
        return chars

    def char_with_spaces_count(self, text):
        return len(text)

    def word_count(self, text):
        return len(nltk.word_tokenize(text))

    def sentence_count(self, text):
        return len(nltk.sent_tokenize(text))

    def paragraph_count(self, text):
        paragraphs = 0
        for c in text:
            if c != "\n": continue
            paragraphs += 1
        return paragraphs

    def page_count(self, text):
        chars_per_page = 3000
        num_chars = len(text)
        return num_chars / chars_per_page

    def for_text(self, text):
        return {
            'size_in_bytes': self.size_in_bytes(text),
            'char_count': self.char_count(text),
            'char_with_spaces_count': self.char_with_spaces_count(text),
            'word_count': self.word_count(text),
            'sentence_count': self.sentence_count(text),
            'paragraph_count': self.paragraph_count(text),
            'page_count': self.page_count(text)
        }



from nltk.tokenize import sent_tokenize, word_tokenize
import string

class JoinLinesSpecification():
    """
    A specification class which acts as a predicate determining if two lines should be joined.

    Methods
    -------
    is_satisfied_by(cand, next_cand)
        Returns True if the two candidate lines should be joined. Otherwise, False.
    """
    def is_satisfied_by(self, cand, next_cand):
        """Returns True if the two candidate lines should be joined. Otherwise, False.

        Parameters
        ----------
        cand : str
            The first line.
        next_cand : str
            The next line.

        Returns
        -------
        bool
            True if the two candidate lines should be joined. Otherwise, False
        """
        next_cand_words = word_tokenize(next_cand)
        if len(next_cand_words) == 0:
            return False
        return cand.endswith(tuple(set(string.punctuation))) is False and (next_cand_words[0].isalpha() and next_cand_words[0].islower())



class MarkdownFormattingService:
    def format_markdown(self, path, values, prefix='{{', suffix='}}'):
        f = open(path, 'r')
        formatted_str = f.read()
        f.close()
        for key, value in values:
            if value is None:
                continue
            old = prefix + key + suffix
            new = str(value)
            formatted_str = formatted_str.replace(old, new)
        return formatted_str



import os
import json

class EUMemberNameService:
    def for_code(self, country_code):
        name = None
        path = os.path.abspath('pygdpr/assets/eu-members.json')
        f = open(path, 'r')
        eu_members_items = json.load(f)
        f.close()
        for cand_code, cand_name in eu_members_items.items():
            if country_code.lower() == cand_code.lower():
                name = cand_name
                break
        return name



from enum import Enum

# Source: https://cloud.google.com/translate/pricing
class TranslateQuotaLimit(Enum):
    UNLIMITED=-1

# https://cloud.google.com/translate/quotas#content
class TranslateQuotaService():
    chars_per_day = 10**9
    chars_per_100_secs = 10**6
    chars_per_100_secs_per_user = 10**4

    def set_chars_per_day(self, number):
        self.chars_per_day = number

    def set_chars_per_100_secs(self, number):
        self.chars_per_100_secs = number

    def set_chars_per_100_secs_per_user(self, number):
        self.chars_per_100_secs_per_user = number

    def get_chars_per_day(self):
        return self.chars_per_day

    def get_chars_per_100_secs(self):
        return self.chars_per_100_secs

    def get_chars_per_100_secs_per_user(self):
        return self.chars_per_100_secs_per_user



import os
import math
import time

import requests
import json
import datetime
import hashlib
import dateparser
import re
import csv
from pygdpr.models.dpa import DPA
from bs4 import BeautifulSoup
from pygdpr.services.filename_from_path_service import filename_from_path_service
from pygdpr.services.pdf_to_text_service import PDFToTextService
from pygdpr.specifications import pdf_file_extension_specification
from pygdpr.specifications.should_retain_document_specification import ShouldRetainDocumentSpecification
from pygdpr.models.common.pagination import Pagination
from pygdpr.policies.gdpr_policy import GDPRPolicy
import textract

class Netherlands(DPA):
    def __init__(self, path=os.curdir):
        country_code='nl'
        super().__init__(country_code, path)

    def update_pagination(self, pagination=None, page_soup=None, driver=None):
        source = {
            'host': 'https://autoriteitpersoonsgegevens.nl',
            'start_path': '/nl/wetgevingsadviezen',
        }
        host = source['host']
        start_path = source['start_path']
        if pagination is None:
            pagination = Pagination()
            pagination.add_item(host + start_path)
        else:
            pager = page_soup.find('div', class_='pager')
            if pager is not None:
                ul_pager = pager.find('ul')
                for li in ul_pager.find_all('li'):
                    page_link = li.find('a')
                    if page_link is None:
                        continue
                    page_href = page_link.get('href')
                    pagination.add_item(host + page_href)
        return pagination

    def get_source(self, page_url=None, driver=None):
        assert (page_url is not None)
        results_response = None
        try:
            results_response = requests.request('GET', page_url)
            results_response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            if to_print:
                print(error)
            pass
        return results_response

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        added_docs = []
        # call all the get_docs_X() functions
        added_docs += self.get_docs_Decisions(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Reports(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Opinions(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_publicDisclosure(existing_docs=[], overwrite=False, to_print=True)
        return added_docs

    def get_docs_Decisions(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Netherlands Decision Documents =========================")
        existed_docs = []
        iteration = 1
        source = {
            'host': 'https://autoriteitpersoonsgegevens.nl',
            'start_path': '/nl/publicaties/boetes-en-sancties',
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        main_content = results_soup.find('div', class_='main-content-article')
        for ul in main_content.find_all('ul'):
            for li in ul.find_all('li'):
                time.sleep(2)
                # get the date: strip all the irrelevant char from the day and year
                date_str_list = li.get_text().split()[-3:]
                new_date = ''
                new_year = ''
                for i in range(len(date_str_list[0])):
                    if date_str_list[0][i].isdigit():
                        new_date += date_str_list[0][i]
                for i in range(len(date_str_list[2])):
                    if date_str_list[2][i].isdigit():
                        new_year += date_str_list[2][i]
                date_str = new_date + ' ' + date_str_list[1] + ' ' + new_year

                tmp = dateparser.parse(date_str, languages=[self.language_code])

                # one exception
                if tmp == None:
                    date_str = "16 juli 2020"
                    tmp = dateparser.parse(date_str, languages=[self.language_code])

                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue

                print('\n------------ Document: ' + str(iteration) + '-------------')
                iteration += 1

                document_title = li.find('a').get_text()
                if date_str == "16 juli 2020":
                    document_title = li.get_text().split('575.000')[0]

                print("\tDocument:\t", document_title)
                document_href = li.find('a').get('href')
                document_url = host + document_href
                if document_href.startswith('https'):
                    document_url = document_href
                print("\tDocument_url:\t", document_url)
                print('\tdate:\t', date)

                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                print('\tdocument_hash:\t', document_hash)
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Decisions' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                    file_response = None
                    try:
                        file_response = requests.request('GET', document_url)
                        file_response.raise_for_status()
                    except requests.exceptions.HTTPError as error:
                        if to_print:
                            print(error)
                        pass
                    if file_response is None:
                        continue
                    if len(file_response.text) == 0:
                        continue

                    file_soup = BeautifulSoup(file_response.text, 'html.parser')

                    # get document full pdf
                    content_bar = file_soup.find_all('div', id='side-content-publications')
                    if content_bar is not None:
                        document_pdf_iter = 1
                        for j in range(len(content_bar)):
                            article_list = content_bar[j].find('ul', class_='article-list')
                            total_document_pdf = article_list.find_all('li')
                            for i in range(0, len(total_document_pdf)):
                                file_url = total_document_pdf[i].find('a').get('href')
                                try:
                                    file_response = requests.request('GET', file_url)
                                    file_response.raise_for_status()
                                except requests.exceptions.HTTPError as error:
                                    if to_print:
                                        print(error)
                                    pass
                                if file_response is None:
                                    continue
                                if len(total_document_pdf) == 1:
                                    document_path = document_folder + '/' + self.language_code
                                else:
                                    document_path = document_folder + '/' + self.language_code + '_' + str(
                                        document_pdf_iter)
                                with open(document_path + '.pdf', 'wb') as f:
                                    f.write(file_response.content)

                                with open(document_path + '.txt', 'wb') as f:
                                    try:
                                        file_text = textract.process(
                                            document_path + '.pdf')
                                        f.write(file_text)
                                    except:
                                        pass
                                document_pdf_iter += 1

                    elif date_str == "16 juli 2020":
                        main_content = file_soup.find('div', class_='rs-panel rs-panel-type-1')
                        main_text = main_content.get_text()
                        with open(
                                document_folder + '/' + self.language_code + '.txt', 'w') as f:
                            f.write(main_text)
                    # if don't have full pdf, get the document summary
                    else:
                        main_content = file_soup.find('div', class_='main-content-article')
                        main_text = main_content.get_text()
                        with open(
                                document_folder + '/' + self.language_code + '_summary.txt', 'w') as f:
                            f.write(main_text)

                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
        return existed_docs

    def get_docs_Reports(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Netherlands Reports =========================")
        iteration = 1
        existed_docs = []
        source = {
            'host': 'https://autoriteitpersoonsgegevens.nl',
            'start_path': '/nl/publicaties/rapportages',
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        main_content = results_soup.find('div', class_='main-content-article')
        for ul in main_content.find_all('ul'):
            for li in ul.find_all('li'):
                year = li.get_text().split()[-1]
                if int(year) < 2018:
                    continue

                print('\n------------ Document: ' + str(iteration) + '-------------')
                iteration += 1
                document_title = li.find('a').get_text()
                print("\tDocument:\t", document_title)
                document_href = li.find('a').get('href')
                document_url = host + document_href
                print("\tDocument_url:\t", document_url)
                print('\tdate:\t', year)
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Reports' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        document_text = document_text.strip()
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': document_title,
                            'md5': document_hash,
                            'releaseDate': year,
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
        return existed_docs

    def get_docs_Opinions(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Netherlands Opinions =========================")
        iteration = 1
        existed_docs = []
        pagination = self.update_pagination()
        # s0. Pagination
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('Page:\t', page_url)
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert results_soup
            article_list = results_soup.find('ul', class_='article-list')
            assert article_list
            # s1. Results
            for li in article_list.find_all('li'):
                date_span = li.find('span', class_='date')
                assert date_span
                date_str = date_span.get_text()
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue

                result_link = li.find('a', class_='download')
                assert result_link
                linktitle = li.find('span', class_='linktitle')
                assert linktitle
                # s2. Documents
                print('\n------------ Document: ' + str(iteration) + '-------------')
                iteration += 1

                document_title = linktitle.get_text().strip()
                print('\tdocument_title:\t', document_title)
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                document_href = result_link.get('href')
                assert document_href
                if document_href.endswith('.pdf') is False:
                    continue

                document_url = document_href
                print('\tdocument_url:\t', document_url)
                if to_print:
                    print("\tDocument:\t", document_hash)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Opinions' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        document_text = document_text.strip()
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': document_title,
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                except FileExistsError:
                    print("Directory path already exists, continue.")

            # s1. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs

    def get_docs_publicDisclosure(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Netherlands public disclosure =========================")
        iteration = 1
        existed_docs = []
        source = {
            'host': 'https://autoriteitpersoonsgegevens.nl',
            'start_path': '/nl/publicaties/wob-besluiten',
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        main_content = results_soup.find('div', class_='main-content-article')
        default_year_list = ['2021', '2020', '2019', '2018']
        for ul in main_content.find_all('ul'):
            default_year = ul.find('li').get_text().split()[-1]
            for li in ul.find_all('li'):
                li_list = li.get_text().split()
                date_str = ' '.join(li_list[-3:])
                # check whether date_str is a valid date
                if not date_str[0].isdigit() or not date_str[-1].isdigit():
                    # check whether date is title's other position
                    # if not in other position, then use the default_date: '05 mei ' + default_year
                    for i in range(len(li_list)):
                        if li_list[i] in default_year_list:
                            date_str = ' '.join(li_list[i - 2:i + 1])
                            break
                        else:
                            date_str = '05 mei ' + default_year
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue

                print('\n------------ Document: ' + str(iteration) + '-------------')
                iteration += 1

                document_title = li.find('a').get_text()
                print("\tDocument:\t", document_title)
                document_href = li.find('a').get('href')
                document_url = host + document_href
                print("\tDocument_url:\t", document_url)
                print("\tdate_str:\t", date_str)

                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print("\tThe error is: ", error)
                    pass
                if document_response is None:
                    continue
                file_soup = BeautifulSoup(document_response.text, 'html.parser')
                if file_soup.find('div', class_='main-content-article') is not None:
                    main_text = file_soup.find('div', class_='main-content-article').get_text()
                    if "kon niet worden gevonden" in main_text:
                        print("\t404 Error, Not found for url")
                        continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Public Disclosure' + '/' + document_hash
                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        document_text = document_text.strip()
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': document_title,
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
        return existed_docs


import os
import math
import requests
import json
import datetime
import hashlib
import dateparser
from pygdpr.models.dpa import DPA
from bs4 import BeautifulSoup
from pygdpr.services.filename_from_path_service import filename_from_path_service
from pygdpr.services.pdf_to_text_service import PDFToTextService
from pygdpr.specifications import pdf_file_extension_specification
from pygdpr.specifications.should_retain_document_specification import ShouldRetainDocumentSpecification
from pygdpr.models.common.pagination import Pagination
from pygdpr.policies.gdpr_policy import GDPRPolicy
import textract

class Hungary(DPA):
    def __init__(self, path=os.curdir):
        country_code='HU'
        super().__init__(country_code, path)

    def update_pagination(self, pagination=None, page_soup=None, driver=None, start_path="decisions"):
        source = {
            "host": "https://www.naih.hu",
            "start_path_decisions": "/hatarozatok-vegzesek?start=0",
            "start_path_recommendations": "/adatvedelmi-ajanlasok",
            "start_path_notices": "/dontesek-adatvedelem-tajekoztatok-koezlemenyek",
            "start_path_resolutions": "/adatvedelmi-allasfoglalasok?start=0",
            "start_path_annualReports": "/eves-beszamolok"
        }
        host = source['host']
        if start_path == "recommendations":
            start_path = source['start_path_recommendations']
        elif start_path == "notices":
            start_path = source['start_path_notices']
        elif start_path == "resolutions":
            start_path = source['start_path_resolutions']
        elif start_path == "annualReports":
            start_path = source['start_path_annualReports']
        else:
            start_path = source['start_path_decisions']
        if pagination is None:
            pagination = Pagination()
            pagination.add_item(host + start_path)
        else:
            pagination = Pagination()
            sectiontablefooter = page_soup.find('div', class_='sectiontablefooter')
            if sectiontablefooter is None:
                return pagination
            pagination_next = sectiontablefooter.find('li', class_='pagination-next')
            if pagination_next is None:
                return pagination
            try_next_link = pagination_next.find('a', class_='hasTooltip pagenav')
            if try_next_link is None:
                return pagination
            next_link = try_next_link.get('href')
            print("next_link:", next_link)
            pagination.add_item(host + next_link)
            #print('added link to pagination: ', host + next_link)
        return pagination



    def get_source(self, page_url=None, driver=None, to_print=True):
        assert (page_url is not None)
        results_response = None
        try:
            results_response = requests.request('GET', page_url)
            results_response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            if to_print:
                print(error)
            pass
        return results_response

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        added_docs = []
        # call all the get_docs_X() functions
        added_docs += self.get_docs_Decisions(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Recommendations(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Notices(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Resolutions(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_AnnualReports(existing_docs=[], overwrite=False, to_print=True)
        return added_docs

    def get_docs_Decisions(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n========================= Hungary Decision Documents ===========================")
        iterator = 1
        existed_docs = []
        dict_hash = {}
        pagination = self.update_pagination()
        # s0. Pagination
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('Page:\t', page_url)
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert results_soup
            pd_category = results_soup.find('div', class_='pd-category')
            for pd_filebox in pd_category.find_all('div', class_='pd-filebox'):

                print("\n------------ Document " + str(iterator) + " ------------")
                iterator += 1

                pd_filenamebox = pd_filebox.find('div', class_='pd-filenamebox')
                pd_filename = pd_filenamebox.find('div', class_='pd-filename')
                pd_float = pd_filename.find('div', class_='pd-float')
                document_title = pd_float.find('a').get_text()
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                document_href = pd_float.find('a').get('href')
                pd_fl_m = pd_filebox.find('div', class_='pd-fl-m')
                date_str = pd_fl_m.get_text()
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                print('\tDocument Title:\t', document_title)
                print('\tdate:\t', date)
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                if document_hash in dict_hash and dict_hash[document_hash] == date:
                    if to_print:
                        print('\tDocument exist:\t', document_hash)
                    continue
                host = "https://www.naih.hu"
                document_url = host + '/' + document_href
                if to_print:
                    print("\tDocument:\t", document_hash)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Decisions' + '/' + document_hash
                # document_folder = dpa_folder + '/hungary' + '/' + 'Decisions' + '/' + document_hash
                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                    dict_hash[document_hash] = date
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs


    def get_docs_Recommendations(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n========================= Hungary Recommendations ===========================")
        iterator = 1
        existed_docs = []
        dict_hash = {}
        pagination = self.update_pagination(start_path="recommendations")
        # s0. Pagination
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('Page:\t', page_url)
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert results_soup
            pd_category = results_soup.find('div', class_='pd-category')
            for pd_filebox in pd_category.find_all('div', class_='pd-filebox'):
                print("\n------------ Document " + str(iterator) + " ------------")
                iterator += 1

                pd_filenamebox = pd_filebox.find('div', class_='pd-filenamebox')
                pd_filename = pd_filenamebox.find('div', class_='pd-filename')
                pd_float = pd_filename.find('div', class_='pd-float')
                document_title = pd_float.find('a').get_text()
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                document_href = pd_float.find('a').get('href')
                pd_fl_m = pd_filebox.find('div', class_='pd-fl-m')
                date_str = pd_fl_m.get_text()
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                print('\tDocument Title:\t', document_title)
                print('\tdate:\t', date)
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                if document_hash in dict_hash and dict_hash[document_hash] == date:
                    if to_print:
                        print('\tDocument exist:\t', document_hash)
                    continue
                host = "https://www.naih.hu"
                document_url = host + '/' + document_href
                if to_print:
                    print("\tDocument:\t", document_hash)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Recommendations' + '/' + document_hash
                # document_folder = dpa_folder + '/hungary' + '/' + 'Recommendations' + '/' + document_hash
                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                    dict_hash[document_hash] = date
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs


    def get_docs_Notices(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n========================= Hungary Notices ===========================")
        iterator = 1
        existed_docs = []
        dict_hash = {}
        pagination = self.update_pagination(start_path="notices")
        # s0. Pagination
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('Page:\t', page_url)
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert results_soup
            pd_category = results_soup.find('div', class_='pd-category')
            for pd_filebox in pd_category.find_all('div', class_='pd-filebox'):

                print("\n------------ Document " + str(iterator) + " ------------")
                iterator += 1

                pd_filenamebox = pd_filebox.find('div', class_='pd-filenamebox')
                pd_filename = pd_filenamebox.find('div', class_='pd-filename')
                pd_float = pd_filename.find('div', class_='pd-float')
                document_title = pd_float.find('a').get_text()
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                document_href = pd_float.find('a').get('href')
                pd_fl_m = pd_filebox.find('div', class_='pd-fl-m')
                date_str = pd_fl_m.get_text()
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                print('\tDocument Title:\t', document_title)
                print('\tdate:\t', date)
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                if document_hash in dict_hash and dict_hash[document_hash] == date:
                    if to_print:
                        print('\tDocument exist:\t', document_hash)
                    continue
                host = "https://www.naih.hu"
                document_url = host + '/' + document_href
                if to_print:
                    print("\tDocument:\t", document_hash)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Notices' + '/' + document_hash
                # document_folder = dpa_folder + '/hungary' + '/' + 'Notices' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                    dict_hash[document_hash] = date
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs


    def get_docs_Resolutions(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n========================= Hungary Resolutions ===========================")
        iterator = 1
        existed_docs = []
        dict_hash = {}
        pagination = self.update_pagination(start_path="resolutions")
        # s0. Pagination
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('Page:\t', page_url)
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert results_soup
            pd_category = results_soup.find('div', class_='pd-category')
            for pd_filebox in pd_category.find_all('div', class_='pd-filebox'):
                print("\n------------ Document " + str(iterator) + " ------------")
                iterator += 1

                pd_filenamebox = pd_filebox.find('div', class_='pd-filenamebox')
                pd_filename = pd_filenamebox.find('div', class_='pd-filename')
                pd_float = pd_filename.find('div', class_='pd-float')
                document_title = pd_float.find('a').get_text()
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                document_href = pd_float.find('a').get('href')
                pd_fl_m = pd_filebox.find('div', class_='pd-fl-m')
                date_str = pd_fl_m.get_text()
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                print('\tDocument Title:\t', document_title)
                print('\tdate:\t', date)
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                if document_hash in dict_hash and dict_hash[document_hash] == date:
                    if to_print:
                        print('\tDocument exist:\t', document_hash)
                    continue
                host = "https://www.naih.hu"
                document_url = host + '/' + document_href
                if to_print:
                    print("\tDocument:\t", document_hash)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Resolutions' + '/' + document_hash
                # document_folder = dpa_folder + '/hungary' + '/' + 'Resolutions' + '/' + document_hash
                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                    dict_hash[document_hash] = date
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs


    def get_docs_AnnualReports(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n========================= Hungary Annual Reports ===========================")
        iterator = 1
        existed_docs = []
        dict_hash = {}
        pagination = self.update_pagination(start_path='annualReports')
        # s0. Pagination
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('Page:\t', page_url)
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert results_soup
            pd_category = results_soup.find('div', class_='pd-category')
            for pd_filebox in pd_category.find_all('div', class_='pd-filebox'):
                print("\n------------ Document " + str(iterator) + " ------------")
                iterator += 1

                pd_filenamebox = pd_filebox.find('div', class_='pd-filenamebox')
                pd_filename = pd_filenamebox.find('div', class_='pd-filename')
                pd_float = pd_filename.find('div', class_='pd-float')
                document_title = pd_float.find('a').get_text()
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                document_href = pd_float.find('a').get('href')
                pd_fl_m = pd_filebox.find('div', class_='pd-fl-m')
                date_str = pd_fl_m.get_text()
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                print('\tDocument Title:\t', document_title)
                print('\tdate:\t', date)
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                if document_hash in dict_hash and dict_hash[document_hash] == date:
                    if to_print:
                        print('\tDocument exist:\t', document_hash)
                    continue
                host = "https://www.naih.hu"
                document_url = host + '/' + document_href
                if to_print:
                    print("\tDocument:\t", document_hash)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Annual Reports' + '/' + document_hash
                # document_folder = dpa_folder + '/hungary' + '/' + 'Annual Reports' + '/' + document_hash
                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                    dict_hash[document_hash] = date
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs


import os
import math
import random

import requests
import json
import datetime
import hashlib
import dateparser
from pygdpr.models.dpa import DPA, MaxRetriesError
from bs4 import BeautifulSoup
from pygdpr.services.filename_from_path_service import filename_from_path_service
from pygdpr.services.pdf_to_text_service import PDFToTextService
from pygdpr.specifications import pdf_file_extension_specification
from pygdpr.specifications.should_retain_document_specification import ShouldRetainDocumentSpecification
from pygdpr.models.common.pagination import Pagination
from pygdpr.policies.gdpr_policy import GDPRPolicy
from striprtf.striprtf import rtf_to_text
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from pygdpr.policies.webdriver_exec_policy import WebdriverExecPolicy
from urllib.parse import urlparse
from pygdpr.models.document import *
import time
import textract

class France(DPA):
    def __init__(self, path=os.curdir):
        country_code='FR'
        super().__init__(country_code, path)

    def update_pagination(self, pagination=None, page_soup=None, driver=None, start_path="decisions"):
        source = {
            "host": "https://www.cnil.fr",
            "start_path_decisions": "/fr/deliberations",
            "start_path_notices": "/thematique/cnil/mises-en-demeure",
            "start_path_guidelines": "/fr/decisions/lignes-directrices-recommandations-CNIL",
            "start_path_recommendation": "/fr/decisions/lignes-directrices-recommandations-CNIL?field_scald_collection_tid=1466",
            "start_path_reports": "/fr/tag/Rapport+annuel"
        }
        host = source['host']
        if start_path == "decisions":
            start_path = source['start_path_decisions']
        elif start_path == "notices":
            start_path = source['start_path_notices']
        elif start_path == "guidelines":
            start_path = source['start_path_guidelines']
        elif start_path == "recommendations":
            start_path = source['start_path_recommendation']
        else:
            start_path = source['start_path_reports']
        if pagination is None:
            pagination = Pagination()
            pagination.add_item(host + start_path)
        else:
            pager_load_more = page_soup.find('ul', class_='pager-load-more')
            if pager_load_more is not None:
                pager_next = pager_load_more.find('li', class_='pager-next')
                page_link = pager_next.find('a')
                if page_link is not None:
                    page_href = page_link.get('href')
                    pagination = Pagination()
                    pagination.add_item(host + page_href)
        return pagination

    def get_source(self, page_url=None, driver=None):
        assert page_url is not None
        results_response = None
        try:
            results_response = requests.request('GET', page_url, timeout=1000)
            results_response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            pass
        return results_response

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        added_docs = []
        # call all the get_docs_X() functions
        added_docs += self.get_docs_Decisions(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Notices(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Guidelines(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Reports(existing_docs=[], overwrite=False, to_print=True)
        return added_docs

    def get_docs_Decisions(self, existing_docs=[], overwrite=False, to_print=True):
        existed_docs = []
        pagination = self.update_pagination()
        print("\n========================= France Decision Documents ===========================")
        iterator = 1
        dict_hashcode = {}
        '''
        added_docs_set = set()
        url = "https://sandbox-oauth.piste.gouv.fr/api/oauth/token"
        client_id = "6e3f9505-809f-45c6-9f93-02ac69f4b384"
        client_secret = "568524c8-8af7-4b48-9602-853882797fb6"
        #client_id = os.environ["PISTE_CLIENT_ID"]
        #client_secret = os.environ["PISTE_CLIENT_SECRET"]
        assert client_id and client_secret
        data = {
            "6c214ae3d2edc9c49a419f7870fe47f7": "1",
            "grant_type": "client_credentials",
            "client_id": "6e3f9505-809f-45c6-9f93-02ac69f4b384",
            "client_secret": "568524c8-8af7-4b48-9602-853882797fb6",
            #"client_id": os.environ["PISTE_CLIENT_ID"],
            #"client_secret": os.environ["PISTE_CLIENT_SECRET"],
            "scope": "openid"
        }
        payload = '&'.join([k + "=" + v for k, v in data.items()])
        headers = {
            'Content-Type': "application/x-www-form-urlencoded",
            'cache-control': "no-cache"
        }
        response = requests.request("GET", url, data=payload, headers=headers, timeout=1000)
        '''
        while pagination.has_next():
            page_url = pagination.get_next()
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            # s1. Results
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            # print(results_soup)
            assert results_soup
            view_content = results_soup.find('div', class_='view-content')
            assert view_content
            for views_row in view_content.find_all('div', class_='views-row'):
                time.sleep(2)
                result_link = views_row.find('a')
                document_href = result_link.get('href')

                print('\n------------ Document: ' + str(iterator) + ' ------------')
                iterator += 1

                # use selenium to bypass incapsula protection
                exec_path = WebdriverExecPolicy().get_system_path()
                options = webdriver.ChromeOptions()
                options.add_argument('headless')
                driver_doc = webdriver.Chrome(options=options, executable_path=exec_path)
                driver_doc.get(document_href)
                document_soup = BeautifulSoup(driver_doc.page_source, 'html.parser')
                '''
                document_response = None
                try:
                    document_response = requests.request('GET', document_href, timeout=1000)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_soup = BeautifulSoup(document_response.text, 'html.parser')
                # print(document_soup) will show "Request unsuccessful. Incapsula incident ID..."
                '''
                assert document_soup
                # print(document_soup)

                main_container = document_soup.find('div', class_='container main-container')
                if main_container is None:
                    print("\tNo content available in this doc")
                    continue
                head_code_page = main_container.find('div', class_='head-code-page')
                assert head_code_page

                if head_code_page.find('h1', class_='main-title') is None:
                    print("\tNo content available in this doc")
                    continue
                document_title = head_code_page.find('h1', class_='main-title').get_text()
                print("\tDocument Title", document_title)
                print("\tdocument_href: ", document_href)

                date_text = document_title.split()[-3:]
                date_str = ' '.join(date_text)
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                # print('date:', tmp.year, tmp.month, tmp.day)
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    print("Before GDPR , stop.")
                    # break
                    return existed_docs
                print("\tdate: ", date)

                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite is False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue

                # documents have the same hashcode, but different dates
                date_part = str(date).replace('-', '_')
                if document_hash in dict_hashcode:
                    document_hash = document_hash + '-' + date_part

                print('\tdocument_hash: ', document_hash)

                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Decisions & Deliberations' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                    main_content = document_soup.find('div', class_='main-col cnil')
                    main_text = main_content.get_text()
                    # print(main_text)
                    with open(
                            document_folder + '/' + self.language_code + '.txt', 'w') as f:
                        f.write(main_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_href
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                    dict_hashcode[document_hash] = date_part
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
                
            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs

    def get_docs_Notices(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n========================= France Notices ===========================")
        iterator = 1
        existed_docs = []
        pagination = self.update_pagination(start_path="notices")
        while pagination.has_next():
            page_url = pagination.get_next()
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            # s1. Results
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            # print(results_soup)
            assert results_soup
            view_content = results_soup.find('div', class_='view-content')
            assert view_content
            for views_row in view_content.find_all('div', class_='views-row'):
                time.sleep(2)
                print('\n------------ Document: ' + str(iterator) + ' ------------')
                iterator += 1

                result_link = views_row.find('a')
                document_href = result_link.get('href')
                host = "https://www.cnil.fr"
                document_url = host + document_href

                document_title = result_link.get_text()
                print("\tDocument Title: ", document_title)
                print("\tDocument URL: ", document_url)
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                document_response = None
                try:
                    # document_response = requests.request('GET', document_href, timeout=1000)
                    document_response = requests.get(document_url, timeout=1000)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_soup = BeautifulSoup(document_response.text, 'html.parser')
                assert document_soup

                main_wrapper = document_soup.find('div', class_='main-wrapper')
                content = main_wrapper.find('div', class_='region region-content')
                date_str = content.find('div', class_='ctn-gen-auteur').get_text()
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                # print('date:', tmp.year, tmp.month, tmp.day)
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    print("This document was published before the GDPR release")
                    continue
                print("\tdate: ", date)
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Notices' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                    main_text = content.get_text()
                    # print(main_text)
                    with open(
                            document_folder + '/' + self.language_code + '.txt', 'w') as f:
                        f.write(main_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")

            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs

    def get_docs_Guidelines(self, existing_docs=[], overwrite=False, to_print=True):
        existing_docs = []
        existing_docs += self.get_docs_Guidelines_part(existing_docs=[], overwrite=False, to_print=True)
        existing_docs += self.get_docs_Recommendations_part(existing_docs=[], overwrite=False, to_print=True)
        return existing_docs

    def get_docs_Guidelines_part(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n========================= France Guidelines Part1 ===========================")
        existed_docs_guidelines_part = []
        pagination = self.update_pagination(start_path="guidelines")
        while pagination.has_next():
            page_url = pagination.get_next()
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            # s1. Results (guidelines)
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            # print(results_soup)
            assert results_soup
            view_content = results_soup.find('div', class_='view-content')
            assert view_content
            for views_row in view_content.find_all('div', class_='views-row'):
                time.sleep(5)
                result_link = views_row.find('a')
                document_href = result_link.get('href')
                print("document_href: ", document_href)
                document_title = result_link.get_text().strip()
                print("\tdocument_title: ", document_title)
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                document_response = None
                try:
                    document_response = requests.request('GET', document_href)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Guidelines' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': "05/25/2018",
                            'url': document_href
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs_guidelines_part.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs_guidelines_part

    def get_docs_Recommendations_part(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n========================= France Guidelines Part2 ===========================")
        existed_docs_recommendations_part = []
        pagination = self.update_pagination(start_path="recommendations")
        while pagination.has_next():
            page_url = pagination.get_next()
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            # s1. Results (guidelines)
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            # print(results_soup)
            assert results_soup
            view_content = results_soup.find('div', class_='view-content')
            assert view_content
            for views_row in view_content.find_all('div', class_='views-row'):
                time.sleep(5)
                result_link = views_row.find('a')
                document_href = result_link.get('href')

                print("document_href: ", document_href)
                document_title = result_link.get_text().strip()
                print("\tdocument_title: ", document_title)
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                document_response = None
                try:
                    document_response = requests.request('GET', document_href)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Guidelines' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                                'md5': document_hash,
                                'releaseDate': "05/25/2018",
                                'url': document_href
                            }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs_recommendations_part.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs_recommendations_part

    def get_docs_Reports(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n========================= France Reports ===========================")
        existed_docs = []
        pagination = self.update_pagination(start_path="reports")
        while pagination.has_next():
            page_url = pagination.get_next()
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            # s1. Results
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            # print(results_soup)
            assert results_soup
            view_content = results_soup.find('div', class_='view-content')
            assert view_content
            for views_row in view_content.find_all('div', class_='views-row'):
                time.sleep(5)
                result_link = views_row.find('a')
                document_href = result_link.get('href')
                host = "https://www.cnil.fr"
                document_url = host + document_href
                print("document_url: ", document_url)
                document_title = result_link.get_text()
                print("\tdocument_title: ", document_title)
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                document_response = None
                try:
                    # document_response = requests.request('GET', document_href, timeout=1000)
                    document_response = requests.get(document_url, timeout=1000)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_soup = BeautifulSoup(document_response.text, 'html.parser')
                assert document_soup

                main_wrapper = document_soup.find('div', class_='main-wrapper')
                content = main_wrapper.find('div', class_='region region-content')
                date_str = content.find('div', class_='ctn-gen-auteur').get_text()
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                # print('date:', tmp.year, tmp.month, tmp.day)
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    print("This document was published before the GDPR release")
                    continue
                print("\tdate: ", date)
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Reports' + '/' + document_hash
                try:
                    os.makedirs(document_folder)
                
                    main_text = content.get_text()
                    # print(main_text)
                    with open(
                            document_folder + '/' + self.language_code + '.txt', 'w') as f:
                        f.write(main_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")

            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs